import matplotlib.pyplot as plt
import missingno as msno
import numpy as np

# import pandas_profiling # this causes segfault!
import plotnine as p9
import scipy as sp
import streamlit as st
from pandaslearn import pd
from sklearn.ensemble import RandomForestRegressor
from sklearn.model_selection import TimeSeriesSplit
from streamlit_pandas_profiling import st_profile_report


def display_data(df):
    st.markdown(f"**{df.shape[0]}** rows and **{df.shape[1]}** columns")
    st.write(df)
    col1, col2 = st.beta_columns(2)
    with col1:
        st.header("Numeric variables")
        numerics = ""
        for item in df.ml.numerical_ix:
            numerics += f"\n* {item}"
        st.markdown(numerics)

    with col2:
        st.header("Categorical variables")
        categoricals = ""
        for item in df.ml.categorical_ix:
            categoricals += f"\n* {item}"
        st.markdown(categoricals)


def display_data_cleanup(df):
    col1, col2 = st.beta_columns(2)
    with col1:
        st.write("Placeholder: column drop recommendations to be displayed later.")
    with col2:
        form = st.form(key="column_drop_form")
        cols = form.multiselect(
            label="Choose variables to be dropped",
            options=list(df.columns),
        )
        submit_button = form.form_submit_button(label="Submit")
        if submit_button:
            df = df.drop(cols, axis=1)
    st.write(f"Column(s) dropped: {cols}")
    st.write(f"Current columns: {list(df.columns)}")
    return df


def display_summary(df):
    colcount = 4
    colcount = st.selectbox("Select number of columns in each table", [3, 4, 5, 6], 1)
    x = (df.ml.numerical_ix.shape[0]) // colcount
    y = (df.ml.numerical_ix.shape[0]) % colcount
    st.write(f"Total {df.ml.numerical_ix.shape[0]} numeric columns.")
    assert x * colcount + y == df.ml.numerical_ix.shape[0]
    for i in range(0, x):
        cols = df.ml.numerical_ix[i * colcount : i * colcount + colcount]
        st.write(df[cols].describe())
    if y != 0:
        cols = df.ml.numerical_ix[x * colcount : x * colcount + y]
        st.write(df[cols].describe())


def display_pandas_profile(df):
    if st.button("Generate pandas-profile report"):
        pr = df.profile_report()
        st_profile_report(pr)


def display_missingvalues(df):
    if st.button("Generate missing value report"):
        p = msno.matrix(df)
        st.pyplot(p.figure)
        q = msno.bar(df)
        st.pyplot(q.figure)
        r = msno.heatmap(df)
        st.pyplot(r.figure)
        s = msno.dendrogram(df)
        st.pyplot(s.figure)


def display_numeric_explorer(df):
    numvar = st.selectbox("Select numeric variable", df.ml.numerical_ix)
    c1, c2 = st.beta_columns((1, 2))
    with c1:
        st.write("Statistial Summary:")
        st.write(df[numvar].describe())
    with c2:
        st.write("Distribution plot:")
        p = (
            p9.ggplot(df, p9.mapping.aes(x=numvar))
            + p9.geoms.geom_histogram(p9.mapping.aes(fill=p9.mapping.after_stat("x")))
            # + p9.coords.coord_flip()
            + p9.themes.theme_xkcd()
            + p9.geoms.geom_text(
                p9.mapping.aes(label=p9.mapping.after_stat("count")),
                stat="count",
                nudge_x=-0.14,
                nudge_y=0.125,
                va="top",
            )
            + p9.geoms.geom_text(
                p9.mapping.aes(label=p9.mapping.after_stat("prop*100"), group=1),
                stat="count",
                nudge_x=0.14,
                nudge_y=0.125,
                va="bottom",
                format_string="{:.1f}% ",
            )
        )
        st.pyplot(p9.ggplot.draw(p))


def display_baselinerf(df):
    # TODO: Put training button here
    # TODO: Put st.empty() elements here, to be filled up ;ater after training completes
    n_iter = st.number_input("Choose number of iterations", min_value=1, max_value=1000, value=10)
    st.markdown("---")
    col1, col2 = st.beta_columns(2)
    with col1:
        st.write("Specify hyperparameters (optional)")
        hpdist_rf = {}
        max_features = st.checkbox("Include max_features", True)
        max_samples = st.checkbox("Include max_samples", True)
        n_estimators = st.checkbox("Include n_estimators", True)
        if n_estimators:
            n_estimators_ = st.slider("Choose max value", 1, 500, 100)
        max_depth = st.checkbox("Include max_depth", True)
        if max_depth:
            max_depth_ = st.slider("Choose max value", 1, 200, 40)
        min_samples_leaf = st.checkbox("Include min_samples_leaf", False)
    with col2:
        if max_features:
            hpdist_rf.update({"max_features": sp.stats.uniform(0, 1)})
        if max_samples:
            hpdist_rf.update({"max_samples": sp.stats.uniform(0, 1)})
        if n_estimators:
            hpdist_rf.update({"n_estimators": sp.stats.randint(10, n_estimators_)})
        if max_depth:
            hpdist_rf.update({"max_depth": sp.stats.randint(2, max_depth_)})
        if min_samples_leaf:
            hpdist_rf.update(
                {
                    "min_samples_leaf": sp.stats.randint(1, 15),
                }
            )
        for key, value in hpdist_rf.items():
            st.write(f"{key}: {value.__class__}")

    st.markdown("---")
    st.write("Specify training config (optional)")

    col1, col2 = st.beta_columns(2)
    with col1:
        st.write("placeholder for training_config widgets")

    with col2:
        config_rf = {
            "n_iter": n_iter,
            "param_distributions": hpdist_rf,
            "n_jobs": -1,
            "cv": TimeSeriesSplit(n_splits=3, gap=50),
            "random_state": 42,
            "verbose": 2,
        }
        for key, value in config_rf.items():
            st.write(f"{key}: {value.__class__}")
        st.write(config_rf)

    st.markdown("---")
    st.write("Choose features")
    col1, col2 = st.beta_columns(2)
    with col1:
        y_var = st.selectbox(
            label="Choose target variable for prediction",
            options=list(df.columns),
            index=len(df.columns) - 1,
        )

    with col2:
        x_vars = st.multiselect(
            label="Choose dependent variables to be used for modeling",
            options=list(df.columns),
        )

    st.write(f"x_vars: {x_vars}")
    st.write(f"y_var: {y_var}")
    df = df.loc[:, x_vars + [y_var]]
    st.write(df)

    if st.button("Start training"):
        (
            df.ml.assign_role(role="target", col=y_var)
            .step_normalize(normalizer="MinMaxScaler()")
            .train(
                model=RandomForestRegressor(),
                config=config_rf,
                sampling_strategy="timeseries",
            )
        )

        st.write(f"model performance: {df.ml.model_performance}")

        st.markdown("---")

        preds = df.ml.predict(df.ml.X_holdout)
        outcome = pd.DataFrame(
            {
                "preds": preds,
                "y_test": df.ml.y_holdout,
            }
        )
        outcome.loc[:, "dev_ratio"] = abs(outcome.preds - outcome.y_test) / outcome.y_test
        st.write(outcome)

        fig, ax = plt.subplots()
        plt.title("Moodel performance")
        plt.scatter(outcome["y_test"], outcome["preds"], c="blue")
        st.pyplot(fig)

        forest = df.ml.model.best_estimator_
        importances = forest.feature_importances_
        std = np.std([tree.feature_importances_ for tree in forest.estimators_], axis=0)
        indices = np.argsort(importances)[::-1]

        st.markdown("## Feature ranking:")

        for f in range(len(x_vars)):
            print(f"{f + 1}. feature {indices[f]} ({importances[indices[f]]})")

        fig, ax = plt.subplots()
        # plt.figure()
        plt.title("Feature importances")
        plt.bar(
            range(len(x_vars)),
            importances[indices],
            color="r",
            yerr=std[indices],
            align="center",
        )
        plt.xticks(range(len(x_vars)), indices)
        plt.xlim([-1, len(x_vars)])
        st.pyplot(fig)



import plotnine as p9
import streamlit as st
from pandaslearn import pd

from .stglance import (
    display_baselinerf,
    display_data,
    display_data_cleanup,
    display_missingvalues,
    display_numeric_explorer,
    display_pandas_profile,
    display_summary,
)

st.set_page_config(layout="wide")

st.sidebar.markdown("# stglance sample app")
st.sidebar.markdown("## Upload data")
uploaded_csv = st.sidebar.file_uploader("Upload CSV", type="csv", key="csv")

if uploaded_csv:
    df = pd.read_csv(uploaded_csv)
    # df = df.drop("Sl", axis=1)
    st.sidebar.write(f"{df.shape[0]} rows and {df.shape[1]} columns")

if not uploaded_csv:
    st.write("Please upload a csv file to continue")
    st.stop()

st.sidebar.markdown("---")
task = st.sidebar.radio(
    label="Select Task",
    options=["Exploratory Data Analysis", "Model Training", "Model Evaluation"],
    index=0,
)

if task == "Exploratory Data Analysis":
    expander01 = st.beta_expander("Display data", expanded=False)
    with expander01:
        display_data(df)

    expander01_xx = st.beta_expander("Data cleanup", expanded=False)
    with expander01_xx:
        df = display_data_cleanup(df)

    expander02 = st.beta_expander("Statistical summary (numeric)", expanded=False)
    with expander02:
        display_summary(df)

    expander03 = st.beta_expander("Pandas profile", expanded=False)
    with expander03:
        display_pandas_profile(df)

    expander04 = st.beta_expander("Missing values", expanded=False)
    with expander04:
        display_missingvalues(df)

    expander05 = st.beta_expander("Explore numeric variables", expanded=False)
    with expander05:
        display_numeric_explorer(df)

    expander06 = st.beta_expander("Baseline RF", expanded=False)
    with expander06:
        display_baselinerf(df)

tabular_app = st



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['stglance', 'stglance.tests']

package_data = \
{'': ['*']}

install_requires = \
['backports.weakref>=1.0.post1',
 'backports.zoneinfo>=0.2.1',
 'catboost>=0.26',
 'category-encoders>=2.2.2',
 'joblib>=1.0.1',
 'lightgbm>=3.2.1',
 'loguru>=0.5.3',
 'missingno>=0.5.0',
 'pandas-profiling>=3.0.0',
 'pandas-ta>=0.3.2-beta.0',
 'pandas>=1.2.5',
 'pdpipe>=0.0.53',
 'plotnine>=0.8.0',
 'pydantic>=1.8.2',
 'scikit-learn>=1.0',
 'scikit-lego>=0.6.7',
 'stackprinter>=0.2.5',
 'streamlit',
 'streamlit-pandas-profiling>=0.1.2',
 'xgboost>=1.4.2',
 'yellowbrick>=1.3.post1']

setup_kwargs = {
    'name': 'stglance',
    'version': '0.1.4',
    'description': 'stglance is a small collection of streamlit widgets (for machine learning) that you can include in your streamlit app.',
    'long_description': '# stglance\n\n`stglance` is a small collection of `streamlit` widgets that you can include in your streamlit app.\n\n\n## Setting up for development\n\n1. `poetry install`\n2. `poetry run pre-commit autoupdate`\n',
    'author': 'Soumendra Prasad Dhanee',
    'author_email': 'soumendra@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/soumendra/stglance',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



from .stglance import (
    display_baselinerf,
    display_data,
    display_data_cleanup,
    display_missingvalues,
    display_numeric_explorer,
    display_pandas_profile,
    display_summary,
)
from .ml_explorer import tabular_app
